<?php

class SPM_ShopyMind_Action_SyncNlSubscribers implements SPM_ShopyMind_Interface_Action
{
    private $params;

    public function __construct(SPM_ShopyMind_Model_Scope $scope, $start, $limit, $lastUpdate, $justCount)
    {
        $this->params['start'] = $start;
        $this->params['limit'] = $limit;
        $this->params['lastUpdate'] = $lastUpdate;
        $this->params['justCount'] = $justCount;
        $this->params['scope'] = $scope;
    }

    public function process()
    {
        $customerIds = $this->retrieveSubscriptions();
        return $customerIds;
    }

    public function retrieveSubscriptions()
    {
        // Get table in configuration
        $stores = $this->params['scope']->stores();

        if(is_array($stores) && count($stores)) {
            $store = $stores[0];
        }

        $table = $store->getConfig('shopymind/configuration/syncnewsletterstable');
        $fields = unserialize($store->getConfig('shopymind/configuration/syncnewsletterstablefields'));

        $tableFields = $finalFields = array();
        if(is_array($fields) && count($fields) > 0) {
            foreach($fields as $field) {
                if($field['shopimind'] != '') {
                    $tableFields[$field['shopimind']] = $field['field'];
                    array_push($finalFields, $field['shopimind']);
                }
            }
        }

        $return = array();

        if(count($tableFields) > 0 && count($finalFields) > 0) {
            $query = 'SELECT ' . ($this->params['justCount'] ? 'COUNT(*) AS `nbSubscribers`' : '`' . implode('`, `', $tableFields) . '`') . '
            FROM `' . $table . '` a
            WHERE 1
            AND `' . $tableFields['id_shop'] . '` = ' . $store->getId() . '
            ' . ($this->params['lastUpdate'] && array_key_exists('last_update', $tableFields) ? ' AND a.`' . $tableFields['last_update'] . '` >= "' . $this->params['lastUpdate'] . '"' : '') . '
            ' . ($this->params['justCount'] ? '' : (array_key_exists('last_update', $tableFields) ? 'ORDER BY a.`' . $tableFields['last_update'] . '`' : '')) . '
            ' . ($this->params['limit'] && !$this->params['justCount'] ? ' LIMIT ' . (int)$this->params['start'] . ', ' . (int)$this->params['limit'] : '');

            $resource = Mage::getSingleton('core/resource');
            $readConnection = $resource->getConnection('core_read');
            $results = $readConnection->fetchAll($query);

            if ($results && is_array($results) && sizeof($results)) {
                foreach ($results as $row) {
                    if ($this->params['justCount']) {
                        $return = $row['nbSubscribers'];
                    } else {
                        // Test if there is an e-mail address field
                        if (array_key_exists($tableFields['email'], $row) && !filter_var($row[$tableFields['email']], FILTER_VALIDATE_EMAIL)) {
                            continue;
                        }

                        array_push($return, array(
                            'id_shop' => array_key_exists($tableFields['id_shop'], $row) ? $row[$tableFields['id_shop']] : null,
                            'last_name' => array_key_exists($tableFields['last_name'], $row) ? $row[$tableFields['last_name']] : null,
                            'first_name' => array_key_exists($tableFields['first_name'], $row) ? $row[$tableFields['first_name']] : null,
                            'email' => array_key_exists($tableFields['email'], $row) ? $row[$tableFields['email']] : null,
                            'zipcode' => array_key_exists($tableFields['zipcode'], $row) ? $row[$tableFields['zipcode']] : null,
                            'last_update' => array_key_exists($tableFields['last_update'], $row) ? $row[$tableFields['last_update']] : null,
                            'newsletter' => array_key_exists($tableFields['status'], $row) && $row[$tableFields['status']] == 1 ? $row[$tableFields['status']] : 0,
                        ));
                    }
                }
            }
        }

        return $return;
    }
}
