<?php

class SPM_ShopyMind_Action_SyncVouchers implements SPM_ShopyMind_Interface_Action
{
    private $params;

    public function __construct(SPM_ShopyMind_Model_Scope $scope, $start, $limit, $lastUpdate, $voucherId, $justCount)
    {
        $this->params['start'] = $start;
        $this->params['limit'] = $limit;
        $this->params['lastUpdate'] = $lastUpdate;
        $this->params['voucherId'] = $voucherId;
        $this->params['justCount'] = $justCount;
        $this->params['scope'] = $scope;
    }

    public function process()
    {


        $storeIds = $this->params['scope']->storeIds();
        $return = array();
        /** @var  $appEmulation Mage_Core_Model_App_Emulation */
        $appEmulation = Mage::getSingleton('core/app_emulation');
        foreach ($storeIds AS $storeId) {
            $initialScope = $this->params['scope'];
            $scope = SPM_ShopyMind_Model_Scope::fromShopymindId('store-'.$storeId);
            //echo $scope->getLang();
            $this->params['scope'] = $scope;

            $emulatedEnvironment = $appEmulation->startEnvironmentEmulation($storeId);

            $customervoucherIds = $this->retrieveVouchers($storeId);

            if ($this->params['justCount']) {
                if($customervoucherIds) {
                    $return = $customervoucherIds;
                    break;
                }
            }
            else {
                foreach ($customervoucherIds as $voucher) {
                    foreach($voucher as $coupon) {
                        $voucher_key = $coupon['id_voucher'] . '-' . $coupon['lang'];
                        if (!isset($return[$voucher_key])) $return[$voucher_key] = $coupon;
                    }
                }
            }
            $appEmulation->stopEnvironmentEmulation($emulatedEnvironment);
            $this->params['scope'] = $initialScope;
        }
        return $return;
    }

    public function retrieveVouchers($storeId)
    {
        $helper = Mage::helper('shopymind');
        $emulatedEnvironment = $helper->startEmulatingScope($this->params['scope']);

        $vouchersCollection = Mage::getModel('salesrule/rule')->getCollection();
        $vouchersCollection->getSelect()->join(
            'salesrule_coupon',
            'main_table.rule_id = salesrule_coupon.rule_id',
            array('*')
        )->group('main_table.rule_id');

        $this->params['scope']->restrictVoucherCollection($vouchersCollection);
        //print_r($vouchersCollection->getData());
        if($this->params['lastUpdate'])
            $vouchersCollection->addFieldToFilter('salesrule_coupon.created_at', array('gteq' => $this->params['lastUpdate']));

        $vouchersCollection->setOrder('salesrule_coupon.created_at','ASC');

        //On exclu si aucun code
        $vouchersCollection->addFieldToFilter('salesrule_coupon.code', array('nin' => ""));

        if ($this->params['voucherId']) {
            $vouchersCollection->addFieldToFilter('salesrule_coupon.coupon_id', array('in' => $this->params['voucherId']));
        }

        if ($this->params['limit']) {
            $vouchersCollection->getSelect()->limit($this->params['limit'], $this->params['start']);
        }

        if ($this->params['justCount']) {
            return $vouchersCollection->count();
        }

        $return = array_map(function($voucher) {
            $arrayReturn = array();
            $couponsForRule = $voucher->getCoupons();
            if(is_array($couponsForRule) && count($couponsForRule) > 0) {
                foreach($couponsForRule as $coupon) {
                    // On détermine le type du bon de réduction
                    $typeVoucher = 0;
                    $amount = 0;

                    $conditionsUnserialized = unserialize($voucher->getConditionsSerialized());
                    $actionsUnserialized = unserialize($voucher->getActionsSerialized());

                    $freeShipping = $voucher->getSimpleFreeShipping()== 1;
                    $reduction = (($voucher->getSimpleAction() == 'cart_fixed' || $voucher->getSimpleAction() == 'by_percent' ) && $voucher->getDiscountAmount());
                    $specificRule = (
                        ($actionsUnserialized['type'] != 'salesrule/rule_condition_product_combine' || (isset($actionsUnserialized['conditions']) && is_array($actionsUnserialized['conditions']) && count($actionsUnserialized['conditions'])))
                        ||
                        ($conditionsUnserialized['type'] != 'salesrule/rule_condition_combine' || (isset($conditionsUnserialized['conditions']) && is_array($conditionsUnserialized['conditions']) && (count($conditionsUnserialized['conditions']) > 1 || (count($conditionsUnserialized['conditions']) == 1 && $conditionsUnserialized['conditions'][0]['attribute'] != 'base_subtotal'))))
                    );
                    if(($freeShipping && $reduction) || ($freeShipping && $specificRule) || ($reduction && $specificRule) || $specificRule) {
                        $typeVoucher = 4; // Custom
                        if($voucher->getSimpleAction() == 'cart_fixed') {
                            $amount = $voucher->getDiscountAmount();
                        }
                        elseif($voucher->getSimpleAction() == 'by_percent') {
                            $amount = $voucher->getDiscountAmount();
                        }
                    }
                    elseif($freeShipping) {
                        // Frais de port offerts
                        $typeVoucher = 3;
                    }
                    elseif($voucher->getSimpleAction() == 'cart_fixed') {
                        // Montant
                        $typeVoucher = 2;
                        $amount = $voucher->getDiscountAmount();
                    }
                    elseif($voucher->getSimpleAction() == 'by_percent') {
                        // Pourcentage
                        $typeVoucher = 1;
                        $amount =$voucher->getDiscountAmount();
                    }
                    $minimum_ammount = 0;
                    if(isset($conditionsUnserialized['conditions']) && is_array($conditionsUnserialized['conditions'])) {
                        foreach ($conditionsUnserialized['conditions'] as $condition) {
                            if($condition['attribute'] == 'base_subtotal') {
                                $minimum_ammount = $condition['value'];
                                break;
                            }
                        }
                    }
                    $ruleLabel = ($voucher->getStoreLabel() ? $voucher->getStoreLabel():($voucher->getDescription()?$voucher->getDescription():$voucher->getName()));

                    $arrayReturn[] = array(
                        'id_shop'   => $this->params['scope']->shopyMindId(),
                        'id_voucher' => $coupon->getCouponId(),
                        'code' => $coupon->getCode(),
                        'name' => $ruleLabel,
                        'date_from'=> $voucher->getFromDate(),
                        'date_to'=> $voucher->getToDate(),
                        'active'=>$voucher->getIsActive(),
                        'id_customer_shop'=>0,
                        'date_creation'=>$coupon->getCreatedAt(),
                        'date_update'=>$coupon->getCreatedAt(),
                        'type_voucher'=>$typeVoucher,
                        'amount'=>$amount,
                        'reduction_tax'=>Mage::getStoreConfig('tax/calculation/discount_tax', Mage::app()->getStore()),
                        'reduction_currency'=>$this->params['scope']->currencyCode(),
                        'minimum_amount'=> $minimum_ammount,
                        'minimum_amount_currency'=>$this->params['scope']->currencyCode(),
                        'used' => (int)($voucher->getTimesUsed() > 0),
                        'lang' =>$this->params['scope']->getLang(),
                    );
                }
            }

            return $arrayReturn;
        }, iterator_to_array($vouchersCollection->getIterator()));
        $helper->stopEmulation($emulatedEnvironment);
        return $return;
    }
}
