<?php

class SPM_ShopyMind_DataMapper_Customer
{	
	public $fromOrderDataMapper = false;
	
    public function format($user)
    {		
		if($this->fromOrderDataMapper) {
			$return = array(
				'id_customer' => $user['entity_id'],
			);
		}
		else {
			$subscription = $this->getNewsletterSubscription($user['entity_id']);
			$return = array(
				'id_customer' => $user['entity_id'],
				'shop_id_shop' => $user['store_id'],
				'optin' => $subscription,
				'newsletter' => $subscription,
				'customer_since' => $user['created_at'],
				'customer_update' => $user['updated_at'],
				'last_name' => $user['lastname'],
				'first_name' => $user['firstname'],
				'email' => $user['email'],
				'gender' => (isset($user['gender_id']) && ($user['gender_id'] == 1 || $user['gender_id'] == 2) ? $user['gender_id'] : 0),
				'birthday' => (isset($user['birthday']) ? date('Y-m-d', strtotime($user['birthday'])) : 0),
				'locale' => $this->getUserLocale($user['entity_id'], $user['store_id'], $user['country_code']),
				'date_last_order' => $this->getDateLastOrder($user['entity_id']),
				'nb_order' => $this->countCustomerOrder($user['entity_id'], null),
				'sum_order' => $this->sumCustomerOrder($user['entity_id']),
				'nb_order_year' => $this->countCustomerOrder($user['entity_id'], '1 YEAR'),
				'sum_order_year' => $this->sumCustomerOrder($user['entity_id'], '1 YEAR'),
				'groups' => array($user['group_id']),
				'active' => true,
				'addresses' => $this->customerAddresses($user['entity_id']),
                'custom' => array(),
			);
		}
		
		return $return;
    }

    private function getUserLocale($id_customer, $store_id, $country_code = false)
    {
        $locale_shop = Mage::getStoreConfig('general/locale/code', $store_id);
        $locale_substr = substr($locale_shop, 0, 3);

        if (!$country_code) {
            $customer = Mage::getModel('customer/customer')->load($id_customer);
            $defaultBilling = $customer->getDefaultBillingAddress();
            if ($defaultBilling){
                return $locale_substr . $defaultBilling->getCountry();
            }
        } else{
            return $locale_substr . $country_code;
        }

        $locale_shop = explode('_', $locale_shop);
        $locale_shop = strtolower($locale_shop [0]) . '_00';

        return $locale_shop;
    }

    private function getDateLastOrder($id_customer)
    {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__)) {
            return call_user_func_array(array ('ShopymindClient_CallbackOverride', __FUNCTION__), func_get_args());
        }

        $tablePrefix = Mage::getConfig()->getTablePrefix();
        $read = Mage::getSingleton('core/resource')->getConnection('core_read');
        if (is_numeric($id_customer)) {
            $result = $read->fetchRow('SELECT MAX(`created_at`) AS `created_at`
			FROM `' . $tablePrefix . 'sales_flat_order`
			WHERE `customer_id` = ' . (int)$id_customer . ' AND `base_total_invoiced` IS NOT NULL');
        } else {
            $result = $read->fetchRow('SELECT MAX(`created_at`) AS `created_at`
			FROM `' . $tablePrefix . 'sales_flat_order`
			WHERE `customer_email` = "' . $id_customer . '" AND `base_total_invoiced` IS NOT NULL');
        }
        return isset($result ['created_at']) ? $result ['created_at'] : 0;
    }

    private function getNewsletterSubscription($id_customer)
    {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__)) {
            return call_user_func_array(array ('ShopymindClient_CallbackOverride', __FUNCTION__), func_get_args());
        }

        $tablePrefix = Mage::getConfig()->getTablePrefix();
        $read = Mage::getSingleton('core/resource')->getConnection('core_read');
		
        if (is_numeric($id_customer)) {
            $result = $read->fetchRow('SELECT COUNT(*) AS `subscribed`
			FROM `' . $tablePrefix . 'newsletter_subscriber`
			WHERE `customer_id` = ' . (int)$id_customer . ' AND `subscriber_status` = 1');
        } else {
            $result = $read->fetchRow('SELECT COUNT(*) AS `subscribed`
			FROM `' . $tablePrefix . 'newsletter_subscriber`
			WHERE `subscriber_email` = "' . $id_customer . '" AND `subscriber_status` = 1');
        }
        return isset($result['subscribed']) && $result['subscribed'] > 0 ? true : false;
    }

    private function countCustomerOrder($customerIdOrEmail, $sinceAgo = null)
    {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__)) {
            return call_user_func_array(array ('ShopymindClient_CallbackOverride', __FUNCTION__), func_get_args());
        }

        $tablePrefix = Mage::getConfig()->getTablePrefix();
        $read = Mage::getSingleton('core/resource')->getConnection('core_read');

        $conditions = $this->ordersConditionsForCustomer($customerIdOrEmail, $sinceAgo);
        $query = sprintf(
            'SELECT COUNT(`entity_id`) AS `nbOrder` FROM `' . $tablePrefix . 'sales_flat_order` WHERE %s',
            implode(' AND ', $conditions)
        );
        $result = $read->fetchRow($query);

        return isset($result['nbOrder']) ? $result['nbOrder'] : 0;
    }

    private function sumCustomerOrder($customerIdOrEmail, $sinceAgo = null)
    {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__)) {
            return call_user_func_array(array ('ShopymindClient_CallbackOverride', __FUNCTION__), func_get_args());
        }

        $tablePrefix = Mage::getConfig()->getTablePrefix();
        $read = Mage::getSingleton('core/resource')->getConnection('core_read');

        $conditions = $this->ordersConditionsForCustomer($customerIdOrEmail, $sinceAgo);
        $query = sprintf(
            'SELECT SUM(`base_total_invoiced`*base_to_order_rate) AS `sumOrder` FROM `' . $tablePrefix . 'sales_flat_order` WHERE %s',
            implode(' AND ', $conditions)
        );
        $result = $read->fetchRow($query);

        return isset($result['sumOrder']) ? $result['sumOrder'] : 0;
    }

    private function ordersConditionsForCustomer($customerIdOrEmail, $sinceAgo)
    {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__)) {
            return call_user_func_array(array ('ShopymindClient_CallbackOverride', __FUNCTION__), func_get_args());
        }

        $conditions = array('`base_total_invoiced` IS NOT NULL');
        if (!is_null($sinceAgo)) {
            $conditions[] = '`created_at` >= DATE_SUB("' . date('Y-m-d H:i:s') . '", INTERVAL ' . $sinceAgo . ')';
        }
        if (is_numeric($customerIdOrEmail)) {
            $conditions[] = '`customer_id` = ' . (int)$customerIdOrEmail;
        } else {
            $conditions[] = '`customer_email` = "' . $customerIdOrEmail . '"';
        }
        return $conditions;
    }

    private function customerAddresses($id_customer)
    {
		if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__)) {
            return call_user_func_array(array ('ShopymindClient_CallbackOverride', __FUNCTION__), func_get_args());
        }

		$helper = Mage::helper('shopymind');
        $tablePrefix = Mage::getConfig()->getTablePrefix();
        $read = Mage::getSingleton('core/resource')->getConnection('core_read');
		
		$query = 'SELECT
					a.entity_id AS addressId,
					addr_firstname.value AS firstname,
					addr_lastname.value AS lastname,
					addr_street.value AS street,
					addr_city.value AS city,
					addr_company.value AS company,
					addr_region_code.code AS stateCode,
					addr_region.value AS state,
					addr_zipcode.value AS postalCode,
					addr_country.value AS countryCode,
					addr_telephone.value AS telephone
				FROM
					' . $tablePrefix . 'customer_entity AS c
				INNER JOIN
					' . $tablePrefix . 'customer_address_entity AS a ON a.parent_id = c.entity_id
				LEFT JOIN ' . $tablePrefix . 'customer_address_entity_varchar AS addr_zipcode ON
					a.entity_id = addr_zipcode.entity_id AND
					addr_zipcode.attribute_id = '.$helper->getMagentoAttributeCode('customer_address', 'postcode').'
				LEFT JOIN ' . $tablePrefix . 'customer_address_entity_varchar AS addr_city ON
					a.entity_id = addr_city.entity_id AND
					addr_city.attribute_id = '.$helper->getMagentoAttributeCode('customer_address', 'city').'
				LEFT JOIN ' . $tablePrefix . 'customer_address_entity_varchar AS addr_company ON
					a.entity_id = addr_company.entity_id AND
					addr_company.attribute_id = '.$helper->getMagentoAttributeCode('customer_address', 'company').'
				LEFT JOIN ' . $tablePrefix . 'customer_address_entity_varchar AS addr_country ON
					a.entity_id = addr_country.entity_id AND
					addr_country.attribute_id = '.$helper->getMagentoAttributeCode('customer_address', 'country_id').'
				LEFT JOIN ' . $tablePrefix . 'customer_address_entity_varchar AS addr_firstname ON
					a.entity_id = addr_firstname.entity_id AND
					addr_firstname.attribute_id = '.$helper->getMagentoAttributeCode('customer_address', 'firstname').'
				LEFT JOIN ' . $tablePrefix . 'customer_address_entity_varchar AS addr_lastname ON
					a.entity_id = addr_lastname.entity_id AND
					addr_lastname.attribute_id = '.$helper->getMagentoAttributeCode('customer_address', 'lastname').'
				LEFT JOIN ' . $tablePrefix . 'customer_address_entity_text AS addr_street ON
					a.entity_id = addr_street.entity_id AND
					addr_street.attribute_id = '.$helper->getMagentoAttributeCode('customer_address', 'street').'
				LEFT JOIN ' . $tablePrefix . 'customer_address_entity_varchar AS addr_telephone ON
					a.entity_id = addr_telephone.entity_id AND
					addr_telephone.attribute_id = '.$helper->getMagentoAttributeCode('customer_address', 'telephone').'
				LEFT JOIN ' . $tablePrefix . 'customer_address_entity_varchar AS addr_region ON
					a.entity_id = addr_region.entity_id AND
					addr_region.attribute_id = '.$helper->getMagentoAttributeCode('customer_address', 'region').'
				LEFT JOIN ' . $tablePrefix . 'customer_address_entity_int AS addr_region_id ON
					a.entity_id = addr_region_id.entity_id AND
					addr_region_id.attribute_id = '.$helper->getMagentoAttributeCode('customer_address', 'region_id').'
				LEFT JOIN ' . $tablePrefix . 'directory_country_region AS addr_region_code ON
					addr_region_id.value = addr_region_code.region_id
					
				WHERE '.(is_numeric($id_customer) ? 'c.`entity_id` = '.$id_customer : 'c.`email` = "'.$id_customer.'"');
		
		$results = $read->fetchAll($query);

        $addresses = array();
        if ($results && is_array($results) && sizeof($results)) {
            foreach ($results as $row) {
                if (!$row['addressId']) {
                    continue;
				}
				
				$street = explode("\n", $row['street']);
				
                array_push($addresses, array(
					'id_address' => $row['addressId'],
					'phone1' => $row['telephone'],
					'phone2' => '',
					'company' => $row['company'],
					'address1' => is_array($street) && count($street) > 0 ? $street[0] : '',
					'address2' => is_array($street) && count($street) > 1 ? $street[1] : '',
					'postcode' => $row['postalCode'],
					'city' => $row['city'],
					'other' => '',
					'active' => '',
					'region' => $row['stateCode'],
					'first_name' => $row['firstname'],
					'last_name' => $row['lastname'],
				));
            }
        }

        return $addresses;
    }
}
