<?php

class SPM_ShopyMind_DataMapper_Product
{
    /**
     * @var $helper SPM_ShopyMind_Helper_Data
     */
    private $helper;

    public function __construct()
    {
        $this->helper = Mage::helper('shopymind');
    }

    public function format(Mage_Catalog_Model_Product $product)
    {
        if (!$product->getId()) {
            return array();
        }

        $shopymindData = array_merge(
            $this->formatProductCommonData($product),
            array(
                'id_product' => $product->getId(),
                'name' => $product->getName(),
                'description_short' => $product->getShortDescription(),
                'description' => $product->getDescription(),
                'combinations' => $this->formattedCombinationsOf($product, $product),
                'id_categories' => $product->getCategoryIds(),
                'id_manufacturer' => $this->helper->manufacturerIdOf($product),
                'quantity_remaining' => $product->getStockItem()->getQty(),
                'date_creation' => $this->helper->formatDateTime($product->getCreatedAt()),
                'date_update' => $this->helper->formatDateTime($product->getUpdatedAt()),
                'active' => (bool) $product->getIsSalable(),
                'custom' => array(),
                'images' => array(),
            )
        );

        return $shopymindData;
    }

    public function formatProductWithCombination(Mage_Catalog_Model_Product $product)
    {
        if (!$product->getId()) {
            return array();
        }
		
		$idProduct = $parent = null;
		
        if($product->getTypeId() == "simple" && !$product->isVisibleInSiteVisibility()){
            $parentIds = Mage::getModel('catalog/product_type_grouped')->getParentIdsByChild($product->getId());
            if(!$parentIds)
                $parentIds = Mage::getModel('catalog/product_type_configurable')->getParentIdsByChild($product->getId());
            if(isset($parentIds[0])){
                $combinationId = $product->getId();
                $idProduct = $parentIds[0];
            }
        }
		
		if(!is_null($idProduct)) {
			$parent = Mage::getModel('catalog/product')->load($idProduct);
		}

        $combinationData = array();
        $CombinationFormatter = new SPM_ShopyMind_DataMapper_Combination();
        if ($CombinationFormatter->isApplicable($product)) {
            $combinationData = $CombinationFormatter->formatSelectedCombinationData($product);
        } else {
           $combinationData = array(
				'id_combination' => null,
				'combination_name' => null,
			);
        }

        $product->unsetData('is_salable');
		
        $shopymindData = array_merge(
            $this->formatProductCommonData($product),
            array(
                'id_product' => $product->getId(),
                'name' => $product->getName(),
                'description_short' => $product->getShortDescription(),
                'description' => $product->getDescription(),
                'id_categories' => $product->getCategoryIds(),
                'id_manufacturer' => $this->helper->manufacturerIdOf($product),
                'date_creation' => $this->helper->formatDateTime($product->getCreatedAt()),
                'date_update' => $this->helper->formatDateTime($product->getUpdatedAt()),
                'active' => (bool) $product->getIsSalable(),
                'quantity_remaining' => $product->getStockItem()->getQty(),
            ),
            $combinationData
        );
		
		if($product->getTypeId() == "simple" && !$product->isVisibleInSiteVisibility() && !is_null($parent)){
			$stockItemModel = Mage::getModel('cataloginventory/stock_item');
			
			$shopymindData = array_merge($shopymindData, array(
				'id_product' => $idProduct,
                'id_combination' => $combinationId,
                'product_link' => str_replace(array('call.php', 'testclient.php', 'checkCallback.php'), 'index.php', $this->helper->productUrlOf($parent)),
				'combination_name' => $product->getName(),
				'name' => $parent->getName(),
				'quantity_remaining' => $stockItemModel->loadByProduct($product)->getQty()
			));
		}

        return $shopymindData;
    }

    private function formattedCombinationsOf(Mage_Catalog_Model_Product $product, $parent = null)
    {
        $parentSuperAttributes = $associatedProducts = null;
        if ($product->getTypeId() === Mage_Catalog_Model_Product_Type_Configurable::TYPE_CODE) {
            $parentSuperAttributes = $product->getTypeInstance(true)->getConfigurableAttributesAsArray($product);
        }
		elseif ($product->getTypeId() === Mage_Catalog_Model_Product_Type_Grouped::TYPE_CODE) {
            $associatedProducts = $product->getTypeInstance(true)->getAssociatedProducts($product);
        }
		
        $attributeNames = array('name', 'stock_item', 'price', 'final_price');

        $commonDataFormatter = array($this, 'formatProductCommonData');
        $childAttributesFormatter = array($this, 'childAttributesFormatter');
        $formatter = function (Mage_Catalog_Model_Product $childProduct) use ($commonDataFormatter, $childAttributesFormatter, $parentSuperAttributes, $parent) {
            $stockItemModel = Mage::getModel('cataloginventory/stock_item');
			
			$arrayMerge = array(
				'combination_name' => $childProduct->getName(),
				'id_combination' => $childProduct->getId(),
				'quantity_remaining' => $stockItemModel->loadByProduct($childProduct)->getQty(),
				'values' => call_user_func($childAttributesFormatter, $childProduct, $parentSuperAttributes),
				'default' => 0,
                'images' => array(),
                'custom' => array(),
			);
			$product_type = $childProduct->getProductType();
			// Si le produit n'est pas visible sur le site, on renvoie l'URL du parent
			if(!$childProduct->isVisibleInSiteVisibility()) {
				$rewrite = Mage::getModel('core/url_rewrite');
				$params = array(
        			'_current' => false,
        			'_use_rewrite' => true,
        			'_secure' => true,
        			'_store_to_url' => false,
        			'_nosid' => true
        		);
				$parentId = (!is_null($parent) ? $parent->getId() : null);
				
				if(is_null($parentId)) {
					$arrayMerge['product_link'] = '';
				}
				else {
					$url = $childProduct->getProductUrl();
					$idPath = 'product/'.$parentId;
					$rewrite->loadByIdPath($idPath);
					$parentUrl = Mage::getUrl($rewrite->getRequestPath(), $params);
					$url = ($parentUrl ? $parentUrl : $url);
					$arrayMerge['product_link'] = str_replace('.html/', '.html', $url);
				}
			}
			
            return array_merge(
                call_user_func($commonDataFormatter, $childProduct, $parent),
                $arrayMerge
            );
        };
        return $this->helper->formatCombinationsOfProduct($product, $formatter, $attributeNames, $parentSuperAttributes, $associatedProducts);
    }

    public function formatProductCommonData(Mage_Catalog_Model_Product $product, $parent = null)
    {
        $product = Mage::getModel('catalog/product')->load($product->getId());


        $baseCurrency = Mage::app()->getStore()->getBaseCurrencyCode();
        $storeCurrency = Mage::app()->getStore()->getCurrentCurrency()->getCode();
        $price = $product->getPrice();
        $price_discount = $product->getFinalPrice();

		if($price == $price_discount || !$price_discount) {
			$store_id = Mage::app()->getStore()->getId();
			
			$date = Mage::app()->getLocale()->storeTimeStamp($store_id);
			$price_discount = Mage::getResourceModel('catalogrule/rule')->getRulePrice($date, Mage::app()->getStore($store_id)->getWebsiteId(), '1', $product->getId());
			$price_discount = (is_null($price_discount) || $price_discount == '') ? $price : $price_discount;
			
			if(!is_null($parent)) {
				$price_discount = Mage::getResourceModel('catalogrule/rule')->getRulePrice($date, Mage::app()->getStore($store_id)->getWebsiteId(), '1', $parent->getId());
				$price_discount = (is_null($price_discount) || $price_discount == '') ? $price : $price_discount;
			}
		}
		
        //Set price from quote currency rate
        if($baseCurrency !== $storeCurrency) {
            $price = Mage::helper('directory')->currencyConvert($product->getPrice(), $baseCurrency, $storeCurrency);
            $price_discount = Mage::helper('directory')->currencyConvert($price_discount, $baseCurrency,$storeCurrency);
            $product->setPrice($price);
            $product->setFinalPrice($price_discount);
        }
		
		// On vérifie si la saisie se fait en HT ou en TTC, et si l'affichage se fait en TTC, pour pouvoir convertir le prix du produit
		$priceIncludeTax = Mage::getStoreConfig('tax/calculation/price_includes_tax', Mage::app()->getStore()); 
		$priceDisplayIncludeTax = Mage::getStoreConfig('tax/display/type', Mage::app()->getStore());

		if(!$priceIncludeTax && in_array($priceDisplayIncludeTax, array(2, 3))) {
			$price = Mage::helper('tax')->getPrice($product, $price, true);
			$price_discount = Mage::helper('tax')->getPrice($product, $price_discount, true);
			$product->setPrice($price);
			$product->setFinalPrice($price_discount);
		}

        $return = array(
            'reference' => $product->getSku(),
            'ean13' => $product->getData('ean'),
            'product_link' => str_replace(array('call.php', 'testclient.php', 'checkCallback.php'), 'index.php', $this->helper->productUrlOf($product)),
            'image_link' => $this->helper->productImageUrlOf($product, $parent),
            'price' => $price,
            'price_discount' => $price_discount,
        );
        $product_type = $product->getProductType();
        //If quote data display quote price
        if($quoteItem = $product->getData('quoteItem')) {
            $return['price_discount'] = $quoteItem->getPriceInclTax();
            $return['quantity'] = $quoteItem->getQty();
            $product_type = $quoteItem->getProductType();
        }
		
		if($product_type != 'simple') {
			$p2 = Mage::getModel('catalog/product')->loadByAttribute('sku',$product->getSku());
			
			if(!is_null($p2) && is_object($p2)) {
				$return['image_link'] = $p2->getData('small_image') != 'no_selection' ? $this->helper->productImageUrlOf($p2, $product) : $this->helper->productImageUrlOf($product, $parent);
			}
		}

        // Si le produit n'est pas visible sur le site, on renvoie l'URL du parent
        if(!$product->isVisibleInSiteVisibility() && in_array($product_type,array('grouped','configurable'))) {
            $rewrite = Mage::getModel('core/url_rewrite');
            $params = array(
                '_current' => false,
                '_use_rewrite' => true,
                '_secure' => true,
                '_store_to_url' => false,
                '_nosid' => true
            );
            $arrayOfParentIds = Mage::getSingleton("catalog/product_type_".$product_type)->getParentIdsByChild($product->getId());
            $parentId = (count($arrayOfParentIds) > 0 ? $arrayOfParentIds[0] : null);
            $url = $product->getProductUrl();
            $idPath = 'product/'.$parentId;
            $rewrite->loadByIdPath($idPath);
            $parentUrl = Mage::getUrl($rewrite->getRequestPath(), $params);
            $url = ($parentUrl ? $parentUrl : $url);
            $return['product_link'] = str_replace('.html/', '.html', $url);
        }
		
		if(!$product->isVisibleInSiteVisibility() && $product_type == 'simple') {
			$return['product_link'] = '';
		}
        
        return $return;
    }

    public function childAttributesFormatter(Mage_Catalog_Model_Product $product, $parentSuperAttributes)
    {
        return array_map(function($attribute) use ($product) {
            $label = '';
            foreach($attribute['values'] as $value) {
                if ($product->getData($attribute['attribute_code']) == $value['value_index']) {
                    $label = $value['store_label'];
                    continue;
                }
            }

            return array(
                'name' => $attribute['store_label'],
                'value' => $label
            );
        }, $parentSuperAttributes);
    }
}
