<?php
/**
 * SPM_ShopyMind_Helper_Data
 *
 * @package     SPM_ShopyMind
 * @copyright   Copyright (c) 2014 - ShopyMind SARL (http://www.shopymind.com)
 * @license     New BSD license (http://license.shopymind.com)
 * @author      Couvert Jean-Sébastien <js.couvert@shopymind.com>
 * @version     $Id Data.php 2014-12-17$
 */
class SPM_ShopyMind_Helper_Data extends Mage_Core_Helper_Abstract
{
    const MANUFACTURER_ATTRIBUTE_CODE = 'manufacturer';

    public function formatCustomerQuote(Mage_Sales_Model_Quote $quote)
    {
        $QuoteFormatter = new SPM_ShopyMind_DataMapper_Quote();

        return $QuoteFormatter->format($quote);
    }

    public function getMagentoAttributeCode($model, $attribute_code) {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__)) {
            return call_user_func_array(array ('ShopymindClient_CallbackOverride', __FUNCTION__), func_get_args());
        }

        $eavAttribute = new Mage_Eav_Model_Mysql4_Entity_Attribute();
        return $eavAttribute->getIdByCode($model, $attribute_code);
    }

    public function getUrl($route, $params = array())
    {
        return str_replace(array('call.php', 'testclient.php', 'checkCallback.php'), 'index.php', Mage::getUrl($route, $params));
    }

    public function productUrlOf(Mage_Catalog_Model_Product $product)
    {
        return $product->getProductUrl(false);
    }

    public function productImageUrlOf(Mage_Catalog_Model_Product $product, Mage_Catalog_Model_Product $parent = null)
    {
		$image = ($product->getData('small_image') == '' || is_null($product->getData('small_image'))) && !is_null($parent) ? (string)Mage::getModel('catalog/product_media_config')->getMediaUrl($parent->getData('small_image')) : (string)Mage::getModel('catalog/product_media_config')->getMediaUrl($product->getData('small_image'));
        return $image;
    }

    public function manufacturerIdOf(Mage_Catalog_Model_Product $product)
    {
        return $product->getData(self::MANUFACTURER_ATTRIBUTE_CODE);
    }

    public function formatDateTime($date)
    {
        return date('Y-m-d H:i:s', strtotime($date));
    }

    public function formatCombinationsOfProduct(Mage_Catalog_Model_Product $product, $formatter, $attributeNames = null, $parentSuperAttributes = null, $associatedProducts = null)
    {
        $combinations = array();
        if ($product->getTypeId() === Mage_Catalog_Model_Product_Type_Configurable::TYPE_CODE) {

            $ids = Mage::getModel('catalog/product_type_configurable')->getChildrenIds($product->getId());
            $childProducts = Mage::getModel('catalog/product')
                ->getCollection()
                ->addAttributeToFilter('entity_id', $ids);

            $attributesToJoin = array();
            if (!empty($parentSuperAttributes)) {
                $attributesToJoin = array_map(function($attribute) { return $attribute['attribute_code']; }, $parentSuperAttributes);

                foreach($attributesToJoin as $attributeToJoin) {
                    $childProducts->joinAttribute($attributeToJoin, 'catalog_product/' . $attributeToJoin, 'entity_id', null, 'left');
                }
            }

            $attributeNames = array_merge(
                $attributesToJoin,
                $attributeNames
            );

            if (!empty($attributeNames)) {
                $childProducts->addAttributeToSelect($attributeNames);
            }

            $where = $childProducts->getSelect()->getPart('where');
			if(is_array($where) && array_key_exists(0, $where) && preg_match('#[a-zA-Z0-9]#', $where[0])) {
				$combinations = array_values(array_map(
					$formatter,
					iterator_to_array($childProducts->getIterator())
				));
				
				// On met la première combinaison en défaut
				if(is_array($combinations) && count($combinations > 0)) {
					$combinations[0]['default'] = 1;
				}
			}
        }
		elseif($product->getTypeId() === Mage_Catalog_Model_Product_Type_Grouped::TYPE_CODE) {
			foreach($associatedProducts as $associatedProduct) {
				$combinations[] = $formatter($associatedProduct);
			}
			
			// On sélectionne par défaut la combinaison la moins chère
			if(is_array($combinations) && count($combinations > 0)) {
				$iCheapest = null;
				
				foreach($combinations as $k => $combination) {
					if(is_null($iCheapest)) {
						$iCheapest = $k;
					}
					else {
						if($combinations[$iCheapest]['price_discount'] > $combination['price_discount']) {
							$iCheapest = $k;
						}
					}
				}
				
				$combinations[$iCheapest]['default'] = 1;
			}
		}
		
        return $combinations;
    }

    public function shortId($length = 6)
    {
        $az = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $azr = rand(0, 51);
        $azs = substr($az, $azr, 10);
        $stamp = hash('sha256', time());
        $mt = hash('sha256', mt_rand(5, 20));
        $alpha = hash('sha256', $azs);
        $hash = str_shuffle($stamp . $mt . $alpha);
        $code = ucfirst(substr($hash, $azr, $length));

        return $code;
    }

    public function startEmulatingScope(SPM_ShopyMind_Model_Scope $scope, $storeId = null)
    {
        $appEmulation = Mage::getSingleton('core/app_emulation');

		if ($storeId == null) {
			$storeIds = $scope->storeIds();
			return $appEmulation->startEnvironmentEmulation($storeIds[0]);
		}
		else {
			return $appEmulation->startEnvironmentEmulation($storeId);
		}
    }

    public function stopEmulation($emulatedEnvironment)
    {
        Mage::getSingleton('core/app_emulation')->stopEnvironmentEmulation($emulatedEnvironment);
    }

    /**
     * Get Config value for certain scope.
     *
     * @param  $path
     * @param  $scopeId
     * @param  null $scope
     * @return mixed
     * @throws Mage_Core_Exception
     */
    public function getConfigValueForScope($path, $scopeId, $scope = null)
    {
        if ($scope == 'websites') {
            $configValue = $this->getMageApp()->getWebsite($scopeId)->getConfig($path);
        } else {
            $configValue = Mage::getStoreConfig($path, $scopeId);
        }
        return $configValue;
    }

    /**
     * Get storeId and/or websiteId if scope selected on back end
     *
     * @param  null $storeId
     * @param  null $websiteId
     * @return array
     */
    public function getConfigScopeId($storeId = null, $websiteId = null)
    {
        $scopeArray = array();
        if ($code = Mage::getSingleton('adminhtml/config_data')->getStore()) {
            // store level
            $storeId = Mage::getModel('core/store')->load($code)->getId();
        } elseif ($code = Mage::getSingleton('adminhtml/config_data')->getWebsite()) {
            // website level
            $websiteId = $this->getCoreWebsite()->load($code)->getId();
            $storeId = $this->getMageApp()->getWebsite($websiteId)->getDefaultStore()->getId();
        }
        $scopeArray['websiteId'] = $websiteId;
        $scopeArray['storeId'] = $storeId;
        return $scopeArray;
    }

    /**
     * Create string for current scope with format scope-scopeId.
     *
     * @return array
     */
    public function getCurrentScope()
    {
        $scopeIdArray = $this->getConfigScopeId();
        $scopeArray = array();
        if (isset($scopeIdArray['websiteId'])) {
            $scopeArray['scope'] = 'websites';
            $scopeArray['scope_id'] = $scopeIdArray['websiteId'];
        } elseif (isset($scopeIdArray['storeId'])) {
            $scopeArray['scope'] = 'stores';
            $scopeArray['scope_id'] = $scopeIdArray['storeId'];
        } else {
            $scopeArray['scope'] = 'default';
            $scopeArray['scope_id'] = 0;
        }
        return $scopeArray;
    }

    /**
     * @return Mage_Core_Model_App
     */
    public function getMageApp()
    {
        return Mage::app();
    }

    /**
     * @return int
     */
    protected function getCurrentStoreId()
    {
        $curStore = $this->getMageApp()->getStore()->getId();
        return $curStore;
    }

    /**
     * @param $magentoStoreId
     */
    protected function setCurrentStore($magentoStoreId)
    {
        $this->getMageApp()->setCurrentStore($magentoStoreId);
    }

    /**
     * @return Mage_Core_Model_Resource
     */
    public function getCoreResource()
    {
        return Mage::getSingleton('core/resource');
    }

    /**
     * @return false|Mage_Core_Model_Abstract
     */
    protected function getCoreWebsite()
    {
        return Mage::getModel('core/website');
    }
}
