<?php

/**
 * @loadSharedFixture
 */
class SPM_ShopyMind_Test_Lib_ShopymindClient_Callback_Get_Inactive_Clients extends EcomDev_PHPUnit_Test_Case
{

    public function testIfNoTimezonesAreGivenReturnFalse()
    {
        $this->assertFalse(ShopymindClient_Callback::getInactiveClients('store-1', '2015-01-31', array(), 3));
    }

    public function testCanGetCustomersWithoutOrdersSinceThreeMonths()
    {
        $expected = array(
            array(
                'customer' => array(
                    'id_customer' => '2',
                    'last_name' => 'Perez',
                    'first_name' => 'Gary',
                    'email' => 'gary.perez41@example.com',
                    'gender' => '1',
                    'locale' => '_00',
                    'birthday' => '1990-01-02',
                    'optin' => false,
                    'newsletter' => false,
                    'customer_since' => '0000-00-00 00:00:00',
                    'phone1' => '',
                    'phone2' => '',
                    'date_last_order' => 0,
                    'nb_order' => '0',
                    'sum_order' => 0,
                    'groups' => array('1'),
                    'shop_id_shop' => '1',
                    'nb_order_year' => '0',
                    'sum_order_year' => 0,
                    'addresses' => array(),
                    'active' => true,
                ),
            ),
        );

        $timezones = array(
            array('country' => 'US', 'region' => 'AL'),
            false
        );
        $actual = ShopymindClient_Callback::getInactiveClients('store-1', '2014-11-24', $timezones, 1);

        $this->assertEquals($expected, $actual);
    }

    public function testIfACountryHasNoOrderTheListIsEmpty()
    {
        $actual = ShopymindClient_Callback::getInactiveClients('store-1', '2015-01-31 23:59:59', array(array('country' => 'FR')), 3);

        $this->assertEmpty($actual);
    }

    public function testIfAStoreHasNoOrderTheListIsEmpty()
    {
        $actual = ShopymindClient_Callback::getInactiveClients('store-2', '2015-01-31 23:59:59', array(array('country' => 'US')), 3);

        $this->assertEmpty($actual);
    }

    public function testItCanCountResults()
    {
        $actual = ShopymindClient_Callback::getInactiveClients('store-1', '2014-11-24', array(array('country' => 'US', 'region' => 'AL')), 1, false, true);

        $this->assertEquals(array('count' => 1), $actual);
    }

    public function testItCanRetrieveDataForMutlipleTimezones()
    {
        $actual = ShopymindClient_Callback::getInactiveClients(
            'store-1',
            '2014-11-24',
            array(
                array('country' => 'US', 'region' => 'AL'),
                array('country' => 'US', 'region' => 'AK'),
            ),
            1,
            false,
            true
        );

        $this->assertEquals(array('count' => 2), $actual);
    }

}
