<?php
/**
 * Callback
 *
 * @package     ShopymindClient
 * @copyright   Copyright (c) 2013 - IDVIVE SARL (http://www.idvive.com)
 * @license     New BSD license (http://license.idvive.com)
 * @author      Yannick Dalbin <yannick@idvive.com>
 * @version     $Id Callback.php 2014-12-17$
 */
if (! in_array('Mage', get_declared_classes())) {
    require_once dirname(__FILE__) . '/../../app/Mage.php';
    umask(0);
    Mage::app();
}
if (file_exists(Mage::getBaseDir('base') . '/lib/ShopymindClient/callback_override.php'))
    require_once (Mage::getBaseDir('base') . '/lib/ShopymindClient/callback_override.php');
class ShopymindClient_Callback {
    protected static $appEmulation = false;
    protected static $initialEnvironmentInfo = false;
    const SEARCH_MIN_LENGTH = 3;

    /**
     * @var null|int Current timestamp (to allow simulating time changes from tests)
     */
    public static $now = null;
	
	private static function getWithoutWait($action, $params)
	{
		$url = Mage::getBaseUrl(Mage_Core_Model_Store::URL_TYPE_WEB);
		
		if(substr($url, -1) != '/') {
			$url .= '/';
		}
		
		$url .= 'lib/ShopymindClient/asyncCall.php?action='.$action.'&'.http_build_query($params);

		exec('wget -q "'.$url.'" -O /dev/null > /dev/null 2>&1 &');
	}

    /**
     * Allow to fetch user information from either an id or email address(es)
     *
     * @param mixed|array $idOrEmails If single and numeric, then it's an id otherwise a list of email addresses
     * @return array
     */
    public static function getUser($idOrEmails, $fromQuotes = false) {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__))
            return call_user_func_array(array (
                    'ShopymindClient_CallbackOverride',
                    __FUNCTION__
            ), func_get_args());

        $GetUserAction = new SPM_ShopyMind_Action_GetUser($idOrEmails, $fromQuotes);
        return $GetUserAction->process();
    }

    /**
     * Récupère la liste des status de commandes différents
     *
     * @return array
     */
    public static function getOrdersStatus() {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__))
            return call_user_func_array(array (
                    'ShopymindClient_CallbackOverride',
                    __FUNCTION__
            ), func_get_args());
        $return = array ();
        $mageVersion = Mage::getVersion();
        if (version_compare ($mageVersion, '1.5', '>')) {
            $results = Mage::getModel('sales/order_status')->getCollection()->addFieldToSelect('status')->addFieldToSelect('label')->getData();
        } else {
            $tablePrefix = Mage::getConfig()->getTablePrefix();
            $resource = Mage::getSingleton('core/resource');

            $readConnection = $resource->getConnection('core_read');
            $query = 'SELECT `value` as `status`, `value` as `label`
	         FROM `' . $tablePrefix . 'sales_order_entity_varchar`
	        WHERE `attribute_id` = ' . self::getMagentoAttributeCode('order_status_history', 'status') . '
	        GROUP BY `value`';
            $results = $readConnection->fetchAll($query);
        }
        if ($results && is_array($results) && sizeof($results)) {
            foreach ( $results as $row ) {
                $return [$row['status']] = $row['label'];
            }
        }
        return $return;
    }

    /**
     * Get store customers' timezones
     *
     * Example of expected result:
     * array(
     *     array(
     *         'country_code' => 'US',
     *         'region_code' => 'GA',
     *     )
     * )
     *
     * @param int $id_shop Magento store id
     * @param string|false $lastUpdate Only get timezones for customers create since $lastUpdate
     * @return array
     */
    public static function getTimezones($id_shop, $lastUpdate) {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__))
            return call_user_func_array(array (
                    'ShopymindClient_CallbackOverride',
                    __FUNCTION__
            ), func_get_args());

        $return = array ();
        $tablePrefix = Mage::getConfig()->getTablePrefix();
        $resource = Mage::getSingleton('core/resource');

        $scope = SPM_ShopyMind_Model_Scope::fromShopymindId($id_shop);

        $readConnection = $resource->getConnection('core_read');
        $query = 'SELECT a.`value` as `country_code`, c.`code` as `region_code`
         FROM `' . $tablePrefix . 'customer_entity` AS d
        JOIN `' . $tablePrefix . 'customer_address_entity` e ON(d.`entity_id` = e.`parent_id`)
        LEFT JOIN `' . $tablePrefix . 'customer_address_entity_varchar` a ON(e.`entity_id` = a.`entity_id`)
        LEFT JOIN `' . $tablePrefix . 'customer_address_entity_int` b ON(a.`entity_id` = b.`entity_id`)
        LEFT JOIN `' . $tablePrefix . 'directory_country_region` c ON(c.`region_id` = b.`value`)
        WHERE (b.`attribute_id` = ' . self::getMagentoAttributeCode('customer_address', 'region_id') . ' OR b.`attribute_id` IS NULL) AND ( a.`attribute_id` = ' . self::getMagentoAttributeCode('customer_address', 'country_id') . ')
        ' . ($lastUpdate ? ' AND d.`updated_at` >= "' . $lastUpdate . '"' : '') . '
        AND d.store_id IN ("' . implode('","', $scope->storeIds()) . '")
        GROUP BY country_code,region_code';
        $results = $readConnection->fetchAll($query);
        if ($results && is_array($results) && sizeof($results)) {
            foreach ( $results as $row ) {
                $return [] = $row;
            }
        }
        return $return;
    }


    private static function productsOfCart(SPM_ShopyMind_Model_Scope $scope, $storeId = null, $cartId)
    {
        $helper = Mage::helper('shopymind');
        $emulatedEnvironment = $helper->startEmulatingScope($scope, $storeId);


        $currentCart = Mage::getModel('sales/quote')->load($cartId);

        $resultProducts = $currentCart->getAllVisibleItems();
        if (empty($resultProducts) || !is_array($resultProducts)) {
            return array();
        }
        $DataTransformer = new SPM_ShopyMind_DataMapper_DataTransformer_QuoteItemToProduct();
        //$DataTransformer2 = new SPM_ShopyMind_DataMapper_QuoteItem();
        $ProductMapper = new SPM_ShopyMind_DataMapper_Product();
        $formatter = new SPM_ShopyMind_DataMapper_Pipeline(array(
            array($DataTransformer, 'transform'),
            array($ProductMapper, 'formatProductWithCombination'),
            SPM_ShopyMind_DataMapper_Scope::makeScopeEnricher($scope),
        ));
        $data = $formatter->format($resultProducts);
        $helper->stopEmulation($emulatedEnvironment);

        return $data;
    }


    private static function productsOfOrder(SPM_ShopyMind_Model_Scope $scope, $quoteId)
    {
        $helper = Mage::helper('shopymind');
        $emulatedEnvironment = $helper->startEmulatingScope($scope);
        $resultProducts = Mage::getModel('sales/quote')->load($quoteId)->getAllVisibleItems();
        if (empty($resultProducts) || !is_array($resultProducts)) {
            return array();
        }

        $ProductMapper = new SPM_ShopyMind_DataMapper_Product();
        $formatter = new SPM_ShopyMind_DataMapper_Pipeline(array(
            function (Mage_Sales_Model_Quote_Item $quoteItem) {
                $product = $quoteItem->getProduct();
                $product->setData('quoteItem',$quoteItem);
                return $product;
            },
            array($ProductMapper, 'format'),
            SPM_ShopyMind_DataMapper_Scope::makeScopeEnricher($scope)
        ));
        $data = $formatter->format($resultProducts);

        $helper->stopEmulation($emulatedEnvironment);
        return $data;
    }

    /**
     * @param $orderId
     * @return array
     */
    public static function getShippingNumbersForOrderId($orderId)
    {
        $tracks = Mage::getModel('sales/order')->load($orderId)->getTracksCollection();
        $shippingNumbers = array();
        if ($tracks->count()) {
            foreach ($tracks as $track) {
                $shippingNumbers[] = $track->getNumber();
            }
        }
        return $shippingNumbers;
    }

    /**
     * Génération des codes de réduction
     *
     * @param array $voucherInfos
     * @param array $emails
     * @return array
     */
    public static function generateVouchers($voucherInfos, $emails, $idShop, $dynamicPrefix, $duplicateCode, $codeToGenerate = false)
    {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__))
            return call_user_func_array(array(
                'ShopymindClient_CallbackOverride',
                __FUNCTION__
            ), func_get_args());

        $vouchers = array();
        if ($voucherInfos && is_array($voucherInfos) && $emails && is_array($emails) && sizeof($emails)) {
            $amount = ($voucherInfos['type'] == 'shipping' ? false : $voucherInfos['amount']);
            $amountCurrency = ($voucherInfos['type'] == 'amount' ? $voucherInfos ['amountCurrency'] : false);

            foreach ($emails as $email) {
                $voucher = self::generateVoucher($email['email'], $voucherInfos['type'], $amount, $amountCurrency, $voucherInfos['minimumOrder'], $voucherInfos['nbDayValidate'], $email['description'], $idShop, $dynamicPrefix, $duplicateCode, $codeToGenerate);
                if (!$voucher) {
                    continue;
                }
                $vouchers[$email['email']] = $voucher;
            }
        }

        return $vouchers;
    }

    /**
     * Création code de réduction
     *
     * @param int $id_customer
     * @param int $type
     * @param string $amount
     * @param string $amountCurrency
     * @param string $minimumOrder
     * @param int $nbDayValidate
     * @param string $description
     * @param string $id_shop
     * @param string $dynamicProfix
     * @param int $duplicateCode
     * @return string
     */
    public static function generateVoucher($id_customer, $type, $amount = false, $amountCurrency = false, $minimumOrder = false, $nbDayValidate, $description, $id_shop, $dynamicPrefix, $duplicateCode, $codeToGenerate = false) {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__))
            return call_user_func_array(array (
                'ShopymindClient_CallbackOverride',
                __FUNCTION__
            ), func_get_args());

        $GenerateVoucher = new SPM_ShopyMind_Action_GenerateVoucher($id_customer, $type, $amount, $amountCurrency, $minimumOrder, $nbDayValidate, $description, $id_shop, $dynamicPrefix, $duplicateCode, $codeToGenerate);
        return $GenerateVoucher->process();
    }

    /**
     * Get customer groups
     *
     * The shop id parameter is not used in Magento as Magento
     * does not manage customer groups by store
     *
     * @param int $id_shop Magento store id
     *
     * @return array
     */
    public static function getCustomerGroups($id_shop) {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__))
            return call_user_func_array(array (
                    'ShopymindClient_CallbackOverride',
                    __FUNCTION__
            ), func_get_args());
        $return = array ();
        $results = Mage::getResourceModel('customer/group_collection')->toOptionArray();
        if ($results) {
            foreach ( $results as $row ) {
                $return[$row['value']] = $row['label'];
            }
        }
        return $return;
    }

    /**
     * Récupération des pays
     *
     * @return array
     */
    public static function getCountries() {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__))
            return call_user_func_array(array (
                    'ShopymindClient_CallbackOverride',
                    __FUNCTION__
            ), func_get_args());
        $return = array ();
        $results = Mage::getResourceModel('directory/country_collection')->loadByStore()->toOptionArray();
        if ($results) {
            foreach ( $results as $row ) {
                if ($row['value'])
                    $return[$row['value']] = $row['label'];
            }
        }
        return $return;
    }

    /**
     * Emulation de la langue de la boutique en fonction du store_id
     *
     * @param int $store_id
     * @return boolean void
     */
    public static function startStoreEmulationByStoreId($store_id) {
        Mage::dispatchEvent('shopymind_start_emulation_before', array('storeId' => $store_id));

        if ($store_id == Mage::app()->getStore()->getId())
            return false;
        self::$appEmulation = Mage::getSingleton('core/app_emulation');
        self::$initialEnvironmentInfo = self::$appEmulation->startEnvironmentEmulation($store_id);
    }

    /**
     * Emulation de la langue de la boutique en fonction de l'ISO langue
     *
     * @param string $lang
     * @return boolean void
     */
    public static function startStoreEmulationByIsoLang($lang,$id_shop) {
        if (self::$appEmulation !== false)
            return;
        if (!empty($id_shop)) {
            list($scope, $id_needed) = explode('-', $id_shop);
        }
        $stores = Mage::app()->getStores();

        foreach ( $stores as $store ) {
            $store_id = $store->getId();
            $website_id = $store->getWebsite()->getId();
            $locale_store = Mage::getStoreConfig('general/locale/code', $store_id);
            $iso_lang = substr($locale_store, 0, - 3);
            if(!isset($id_needed) || $id_needed == 0) {
                self::startStoreEmulationByStoreId($store_id);
                break;
            }
            elseif($scope === 'store') {
                if($id_needed == $store_id) {
                    self::startStoreEmulationByStoreId($store_id);
                    break;
                }
            }elseif($scope === 'website') {
                if ($lang === substr($locale_store, 0, - 3) && $id_needed == $website_id) {
                    self::startStoreEmulationByStoreId($store_id);
                    break;
                }
            }
        }
    }

    private static function startAdminStoreEmulation()
    {
        foreach (Mage::app()->getStores(true) as $store) {
            if ($store->isAdmin()) {
                self::startStoreEmulationByStoreId($store->getId());
                break;
            }
        }
    }

    public static function isStoreEmulated()
    {
        return (self::$appEmulation !== false);
    }

    /**
     * Retour à l'environnement normal
     *
     * @param string $lang
     * @return boolean void
     */
    public static function stopStoreEmulation() {
        if (self::$appEmulation === false)
            return;
        self::$appEmulation->stopEnvironmentEmulation(self::$initialEnvironmentInfo);
        self::$appEmulation = false;
        self::$initialEnvironmentInfo = false;
    }

    /**
     * Récupération des devises
     *
     * @return array
     */
    public static function getCurrencies() {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__))
            return call_user_func_array(array (
                    'ShopymindClient_CallbackOverride',
                    __FUNCTION__
            ), func_get_args());
        $return = array ();
        $results = Mage::app()->getStore()->getAvailableCurrencyCodes(true);
        if ($results) {
            foreach ( $results as $value ) {
                if ($value)
                    $return [$value] = Mage::app()->getLocale()->getTranslation($value, 'nametocurrency');
            }
        }
        return $return;
    }

    /**
     * Récupération des langues disponibles
     * @param unknown $id_shop
     * @return mixed|multitype:unknown
     */
    public static function getLangs($id_shop) {
    	if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__))
    		return call_user_func_array(array (
    				'ShopymindClient_CallbackOverride',
    				__FUNCTION__
    		), func_get_args());
    	$return = array ();
    	if (!empty($id_shop)) {
    		list($scope, $id_needed) = explode('-', $id_shop);
    	}
    	$stores = Mage::app()->getStores();

    	foreach ( $stores as $store ) {
    		$store_id = $store->getId();
    		$website_id = $store->getWebsite()->getId();
    		$locale_store = Mage::getStoreConfig('general/locale/code', $store_id);
    		$iso_lang = substr($locale_store, 0, - 3);
    		if(!isset($id_needed) || $id_needed == 0)
    		    $return[$iso_lang] = $iso_lang;
    		elseif($scope === 'store') {
    		    if($id_needed == $store_id) {
    		        $return[$iso_lang] = $iso_lang;
    		        break;
    		    }
    		}elseif($scope === 'website') {
    		    if($id_needed == $website_id) {
    		        $return[$iso_lang] = $iso_lang;
    		    }
    		}
    	}
    	return $return;
    }
    /**
     * Récupération des moyens de livraison
     * @return array
     */
    public static function getCarriers() {
    	if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__))
    		return call_user_func_array(array (
    				'ShopymindClient_CallbackOverride',
    				__FUNCTION__
    		), func_get_args());
    		$return = array ();
    		$results = Mage::getSingleton('shipping/config')->getActiveCarriers();

    		foreach($results as $_code => $_method)
    		{
    			if(!$_title = Mage::getStoreConfig("carriers/$_code/title"))
                    $_title = $_code;
                $return[$_code] = $_title . " ($_code)";
    		}
    		return $return;
    }
    /**
     * Récupération des boutiques
     * @return array
     */
    public static function getShops() {
    	if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__))
    		return call_user_func_array(array (
    				'ShopymindClient_CallbackOverride',
    				__FUNCTION__
    		), func_get_args());
    	$return = array ();
    	$websites = Mage::getModel('core/website')->getCollection();
    	foreach ($websites as $website){
    		$return[$website->getId()] = $website->getName();
    	}
    	return $return;
    }
    /**
     * Get test data
     *
     * @param int $id_shop Magento storeId
     * @param string|bool $lang Locale to use
     *
     * @return array
     */
    public static function getTestData($id_shop, $lang = false) {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__))
            return call_user_func_array(array (
                    'ShopymindClient_CallbackOverride',
                    __FUNCTION__
            ), func_get_args());
        if ($lang)
            self::startStoreEmulationByIsoLang($lang,$id_shop);

        if (!is_null($id_shop)) {
            //$store_id = explode('-', $id_shop);
            $store_id = Mage::app()->getStore()->getId();
        }
        else {
            $store_id = null;
        }

        // Lien vers panier
        $return = array();
        $return ['link_cart'] = Mage::helper('shopymind')->getUrl('checkout/cart', array(
			'_current' => false,
			'_use_rewrite' => true,
			'_secure' => true,
			'_store' => $store_id,
			'_store_to_url' => false,
			'_nosid' => true
		));
        // Article au hasard
        $return ['articles'] = self::getProducts($id_shop, $lang, false, true);
        if ($lang)
            self::stopStoreEmulation();
        return $return;
    }

    /**
     * Supression des bons de réduction ShopyMind périmés
     *
     * @return boolean
     */
    public static function deleteUnusedVouchers() {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__))
            return call_user_func_array(array (
                    'ShopymindClient_CallbackOverride',
                    __FUNCTION__
            ), func_get_args());
        $date = date('Y-m-d H:i:s');
        $tablePrefix = Mage::getConfig()->getTablePrefix();
        $read = Mage::getSingleton('core/resource')->getConnection('core_read');
        $results = $read->fetchAll('SELECT `rule_id` AS `id`
                FROM `' . $tablePrefix . 'salesrule`
                WHERE `name` LIKE "SPM-%" AND `to_date` <= DATE_SUB("' . $date . '", INTERVAL 7 DAY)');
        if ($results && is_array($results) && sizeof($results)) {
            foreach ( $results as $row ) {
                $cartRule = Mage::getModel('salesrule/rule')->load($row['id']);
                $cartRule->delete();
            }
        }
        return true;
    }

    /**
     * Get products list
     * Both $products and $random cannot be false at the same time
     *
     * @param int $id_shop Magento store id
     * @param string $lang Locale to use
     * @param array|bool $products If array, list of product ids to fetch. If set to false, use the $random parameter
     * @param bool $random Fetch random products
     * @param int $maxProducts Maximum number of products to fetch if random
     *
     * @return array
     */
    public static function getProducts($id_shop, $lang, $products = false, $random = false, $maxProducts = 3) {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__)) {
            return call_user_func_array(array (
                    'ShopymindClient_CallbackOverride',
                    __FUNCTION__
            ), func_get_args());
        }
        /** @var SPM_ShopyMind_Helper_Data $helper */
        $helper = Mage::helper('shopymind');

        $scope = SPM_ShopyMind_Model_Scope::fromShopymindId($id_shop, $lang);
        $emulatedEnvironment = $helper->startEmulatingScope($scope);

        $collection = array();
        if ($products) {
            $collection = Mage::getResourceModel('catalog/product_collection');
            $collection
                ->addAttributeToSelect(Mage::getSingleton('catalog/config')->getProductAttributes())
                ->addAttributeToFilter('entity_id', array (
                    'in' => $products
                ))
                ->addAttributeToFilter('status', array(
                        'in' => Mage::getModel('catalog/product_status')->getVisibleStatusIds())
                )
                ->addAttributeToFilter('visibility', array(
                        'in' => Mage::getModel('catalog/product_visibility')->getVisibleInCatalogIds())
                )
                ->setFlag('require_stock_items', true)
                ->getSelect();
            $collection->setPage(1, ($maxProducts ? $maxProducts : 3));
        } elseif ($random) {
            $collection = Mage::getResourceModel('catalog/product_collection');
            $collection
                ->addAttributeToSelect(Mage::getSingleton('catalog/config')->getProductAttributes())
                ->addAttributeToFilter('status', array(
                        'in' => Mage::getModel('catalog/product_status')->getVisibleStatusIds())
                )
                ->addAttributeToFilter('visibility', array(
                        'in' => Mage::getModel('catalog/product_visibility')->getVisibleInCatalogIds())
                )
                ->setFlag('require_stock_items', true)
                ->getSelect()->order('rand()');
            $collection->setPage(1, ($maxProducts ? $maxProducts : 3));
        }
        $scope->restrictProductCollection($collection);

        $ProductDataMapper = new SPM_ShopyMind_DataMapper_Product();
        $formatter = new SPM_ShopyMind_DataMapper_Pipeline(array(
            array($ProductDataMapper, 'format'),
            SPM_ShopyMind_DataMapper_Scope::makeScopeEnricher($scope)
        ));
        $result = $formatter->format(iterator_to_array($collection));

        $helper->stopEmulation($emulatedEnvironment);
        return array_values($result);
    }

    /**
     * Récupération de la locale d'un client
     *
     * @param int|string $id_customer
     * @param int $store_id
     * @param string $country_code
     * @return string
     */
    public static function getUserLocale($id_customer, $store_id, $country_code = false) {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__))
            return call_user_func_array(array (
                    'ShopymindClient_CallbackOverride',
                    __FUNCTION__
            ), func_get_args());
        $locale_shop = Mage::getStoreConfig('general/locale/code', $store_id);
        if (! $country_code) {
            $customer = Mage::getModel('customer/customer')->load($id_customer);
            $defaultBilling = $customer->getDefaultBillingAddress();
            if ($defaultBilling)
                return substr($locale_shop, 0, 3) . $defaultBilling->getCountry();
        } else
            return substr($locale_shop, 0, 3) . $country_code;

        $locale_shop = explode('_', $locale_shop);
        $locale_shop = strtolower($locale_shop [0]) . '_00';

        return $locale_shop;
    }

    /**
     * Récupération du complément de requete de timezone
     *
     * @param array $timezones
     * @param string $country_alias
     * @param string $country_field
     * @param string $state_alias
     * @param string $state_field
     * @return boolean string
     */
    public static function generateTimezonesWhere($timezones, $country_alias = 'customer_default_billing_country', $country_field = 'value', $state_alias = 'customer_default_billing_state', $state_field = 'code') {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__))
            return call_user_func_array(array (
                    'ShopymindClient_CallbackOverride',
                    __FUNCTION__
            ), func_get_args());
        if (is_array($timezones) and ! sizeof($timezones))
            return false;
        $where = array ();
        foreach ( $timezones as $zones ) {
            if (! is_array($zones))
                $where [] = '(`' . $country_alias . '`.`' . $country_field . '` IS NULL)';
            elseif (! isset($zones ['region']) || ! $zones ['region'])
                $where [] = '(`' . $country_alias . '`.`' . $country_field . '` = "' . $zones ['country'] . '")';
            else
                $where [] = '(`' . $country_alias . '`.`' . $country_field . '` = "' . $zones ['country'] . '" AND `' . $state_alias . '`.`' . $state_field . '` = "' . $zones ['region'] . '")';
        }
        if (sizeof($where))
            return '(' . implode(' OR ', $where) . ')';
        return false;
    }

    /**
     * Récupération date de dernière commande d'un client
     *
     * @param int|string $id_customer
     * @return string int
     */
    public static function getDateLastOrder($id_customer) {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__))
            return call_user_func_array(array (
                    'ShopymindClient_CallbackOverride',
                    __FUNCTION__
            ), func_get_args());
        $tablePrefix = Mage::getConfig()->getTablePrefix();
        $read = Mage::getSingleton('core/resource')->getConnection('core_read');
        if (is_numeric($id_customer)) {
            $result = $read->fetchRow('SELECT MAX(`created_at`) AS `created_at`
			FROM `' . $tablePrefix . 'sales_flat_order`
			WHERE `customer_id` = ' . (int) $id_customer . ' AND `base_total_invoiced` IS NOT NULL');
        } else {
            $result = $read->fetchRow('SELECT MAX(`created_at`) AS `created_at`
			FROM `' . $tablePrefix . 'sales_flat_order`
			WHERE `customer_email` = "' . $id_customer . '" AND `base_total_invoiced` IS NOT NULL');
        }
        return isset($result ['created_at']) ? $result ['created_at'] : 0;
    }

    /**
     * Number of orders passed by a customer, optionally on a given period
     *
     * @param int|string $customerIdOrEmail
     * @param string $sinceAgo Period to consider (in SQL DATE_SUB compatible format), optional (example: "1 YEAR")
     * @return int
     */
    public static function countCustomerOrder($customerIdOrEmail, $sinceAgo = null) {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__)) {
            return call_user_func_array(array (
                'ShopymindClient_CallbackOverride',
                __FUNCTION__
            ), func_get_args());
        }

        $tablePrefix = Mage::getConfig()->getTablePrefix();
        $read = Mage::getSingleton('core/resource')->getConnection('core_read');

        $conditions = self::ordersConditionsForCustomer($customerIdOrEmail, $sinceAgo);
        $query = sprintf(
            'SELECT COUNT(`entity_id`) AS `nbOrder` FROM `' . $tablePrefix . 'sales_flat_order` WHERE %s',
            implode(' AND ', $conditions)
        );
        $result = $read->fetchRow($query);

        return isset($result['nbOrder']) ? $result['nbOrder'] : 0;
    }

    /**
     * Total amount ordered by a client, optionally filtered on a given period
     *
     * @param int|string $customerIdOrEmail
     * @param string $sinceAgo Period to consider (in SQL DATE_SUB compatible format), optional (example: "1 YEAR")
     * @return int float
     */
    public static function sumCustomerOrder($customerIdOrEmail, $sinceAgo = null) {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__)) {
            return call_user_func_array(array (
                    'ShopymindClient_CallbackOverride',
                    __FUNCTION__
            ), func_get_args());
        }

        $tablePrefix = Mage::getConfig()->getTablePrefix();
        $read = Mage::getSingleton('core/resource')->getConnection('core_read');

        $conditions = self::ordersConditionsForCustomer($customerIdOrEmail, $sinceAgo);
        $query = sprintf(
            'SELECT SUM(`base_total_invoiced`*base_to_order_rate) AS `sumOrder` FROM `' . $tablePrefix . 'sales_flat_order` WHERE %s',
            implode(' AND ', $conditions)
        );
        $result = $read->fetchRow($query);

        return isset($result['sumOrder']) ? $result['sumOrder'] : 0;
    }

    private static function ordersConditionsForCustomer($customerIdOrEmail, $sinceAgo)
    {
        $conditions = array('`base_total_invoiced` IS NOT NULL');
        if (!is_null($sinceAgo)) {
            $conditions[] = '`created_at` >= DATE_SUB("' . date('Y-m-d H:i:s') . '", INTERVAL ' . $sinceAgo . ')';
        }
        if (is_numeric($customerIdOrEmail)) {
            $conditions[] = '`customer_id` = ' . (int)$customerIdOrEmail;
        } else {
            $conditions[] = '`customer_email` = "' . $customerIdOrEmail . '"';
        }
        return $conditions;
    }

    /**
     * Récupération du code d'un attribut
     *
     * @param string $model
     * @param string $attribute_code
     * @return int
     */
    public static function getMagentoAttributeCode($model, $attribute_code) {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__))
            return call_user_func_array(array (
                    'ShopymindClient_CallbackOverride',
                    __FUNCTION__
            ), func_get_args());
        $eavAttribute = new Mage_Eav_Model_Mysql4_Entity_Attribute();
        return $eavAttribute->getIdByCode($model, $attribute_code);
    }

    /**
     * Récupération de l'id du client en session
     *
     * @return int boolean
     */
    public static function getSessionCustomerId() {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__))
            return call_user_func_array(array (
                    'ShopymindClient_CallbackOverride',
                    __FUNCTION__
            ), func_get_args());
        if (Mage::getSingleton('customer/session')->isLoggedIn()) {
            $customerData = Mage::getSingleton('customer/session')->getCustomer();
            return $customerData->getId();
        }
        return false;
    }

    /**
     * Récupération de l'email du client en session
     *
     * @return string boolean
     */
    public static function getSessionCustomerEmail() {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__))
            return call_user_func_array(array (
                    'ShopymindClient_CallbackOverride',
                    __FUNCTION__
            ), func_get_args());
        if (Mage::getSingleton('customer/session')->isLoggedIn()) {
            $customerData = Mage::getSingleton('customer/session')->getCustomer();
            return $customerData->getEmail();
        } else {
            $quote = Mage::getSingleton('checkout/session')->getQuote();
            return $quote->getCustomerEmail();
        }
        return false;
    }

    /**
     * Récupération de l'id du panier du client en session
     *
     * @return int boolean
     */
    public static function getSessionCartId() {
        $session = Mage::getSingleton('checkout/session');
        $cart_id = $session->getQuote()->getId();
        return $cart_id;
    }

    /**
     * Remontée des ventes vers ShopyMind
     *
     * @param array $order
     */
    /*public static function checkNewOrder($order) {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__))
            return call_user_func_array(array (
                    'ShopymindClient_CallbackOverride',
                    __FUNCTION__
            ), func_get_args());

        self::sendOrderToSPM($order);
    }*/

    /**
     * Envoi des ventes générées suite à une relance à ShopyMind
     *
     * @param Mage_Sales_Model_Order $order
     */
    /*public static function sendOrderToSPM($order) {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__)) {
            return call_user_func_array(array (
                    'ShopymindClient_CallbackOverride',
                    __FUNCTION__
            ), func_get_args());
        }
        self::startStoreEmulationByStoreId($order->getStoreId());

        include_once (Mage::getBaseDir('base') . '/lib/ShopymindClient/Bin/Notify.php');
        $params = self::formatOrderData($order);
        ShopymindClient_Bin_Notify::newOrder($params);
        self::stopStoreEmulation();
    }*/

    public static function saveOrder($order)
    {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__))
            return call_user_func_array(array(
                'ShopymindClient_CallbackOverride',
                __FUNCTION__
            ), func_get_args());

        self::startStoreEmulationByStoreId($order->getStoreId());

        //On vérifie que la commande n'est pas trop ancienne
        $date = new DateTime();
        $date->sub(new DateInterval('P3M'));
        $dateOrder = new DateTime($order->getCreatedAtStoreDate()->toString(Varien_Date::DATETIME_INTERNAL_FORMAT));

        if($date > $dateOrder)
            return;

        $scope = SPM_ShopyMind_Model_Scope::fromShopymindId($order->getStoreId());
        $realtimeEnabled = $scope->getConfig('shopymind/configuration/realtimesync');

        if($realtimeEnabled == 1) {
            $disabledFunctions = explode(',', ini_get('disable_functions'));
            if (is_callable('exec') && !in_array('exec', $disabledFunctions)) {
                self::getWithoutWait('saveOrder', array('id' => $order->getId(), 'store_id' => $order->getStoreId()));
            } else {
                include_once(Mage::getBaseDir('base') . '/lib/ShopymindClient/Bin/Notify.php');
                ShopymindClient_Bin_Notify::saveOrder($order->getId());
            }
        }
        else {
            // Si la synchro temps réel est désactivée par l'admin, on ne synchronise en temps réel que les dernières commandes (-48h)
            $date = new DateTime();
            $date->sub(new DateInterval('P2D'));
            $dateOrder = new DateTime($order->getCreatedAtStoreDate()->toString(Varien_Date::DATETIME_INTERNAL_FORMAT));

            if($date <= $dateOrder) {
                $disabledFunctions = explode(',', ini_get('disable_functions'));
                if (is_callable('exec') && !in_array('exec', $disabledFunctions)) {
                    self::getWithoutWait('saveOrder', array('id' => $order->getId(), 'store_id' => $order->getStoreId()));
                } else {
                    include_once(Mage::getBaseDir('base') . '/lib/ShopymindClient/Bin/Notify.php');
                    ShopymindClient_Bin_Notify::saveOrder($order->getId());
                }
            }
        }
        self::stopStoreEmulation();
    }

    /**
     * Allow to retrieve the Magento customers list
     * This method is used for mailing or SMS campaign
     *
     * @param string $storeId Shopymind store id
     * @param string $start
     * @param int $limit
     * @param string $lastUpdate
     * @param boolean $justCount
     *
     * @return array $customers
     */
    public static function getContacts($storeId, $start, $limit, $lastUpdate, $justCount = false)
    {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__)) {
            return call_user_func_array(array(
                'ShopymindClient_CallbackOverride',
                __FUNCTION__
            ), func_get_args());
        }
        if ($justCount) {
            $count = 0;
        }
        $scope = SPM_ShopyMind_Model_Scope::fromShopymindId($storeId);

        $customerCollection = Mage::getModel('customer/customer')
            ->getCollection()
            ->addFieldToFilter('updated_at', array('gt' => $lastUpdate))
            ->addAttributeToSelect('entity_id');
        $scope->restrictCollection($customerCollection);

        $customerCollection->getSelect()->where('is_active = 1');

        if ($limit) {
            $customerCollection->getSelect()->limit($limit, $start);
        }

        $customers = array();
        foreach($customerCollection as $customer) {
            if ($justCount) {
                $count++;
            } else {
                $customers[] = array (
                    'customer' => self::getUser($customer['entity_id'])
                );
            }
        }

        //Add guest customer
        $guestCollection = Mage::getResourceModel('sales/order_collection')
            ->addAttributeToSelect('customer_email');

        $scope->restrictCollection($guestCollection);

        if (!is_null($lastUpdate)) {
            $guestCollection->addFieldToFilter('updated_at', array('gteq' => $lastUpdate));
        }

        $guestCollection->addFieldToFilter('customer_is_guest', 1);

        if (!is_null($limit)) {
            $guestCollection->getSelect()->limit($limit, $start);
        }


        foreach ($guestCollection as $customer) {
            if ($justCount) {
                $count++;
            } else {
                $customers[] = array (
                    'customer' => self::getUser($customer['customer_email'])
                );
            }
        }

        if ($justCount) {
            return $count;
        }

        return $customers;
    }

    private static function counterResponse(Varien_Data_Collection $collection)
    {
        return array(
            'count' => $collection->count()
        );
    }

    /**
     * Formate les données d'une commande dans le format attendu par le serveur de ShopyMind
     */
    private static function formatOrderData(Mage_Sales_Model_Order $order)
    {
        $OrderDataMapper = new SPM_ShopyMind_DataMapper_Order();

        return $OrderDataMapper->format($order);
    }

    /**
     * Method allowing to do a textual lookup for manufacturers matching a given search query
     *
     * @param $id_shop Id of the shop to restrict results to
     * @param bool $lang Allow to filter results for a store with a specific language
     * @param $search Text to match against manufacturer names. The search must be at least 3 chars long
     * @return array List of manufacturers (array('id' => 'xx', 'name' => 'yy')) ordered alphabetically
     */
    public static function findManufacturers($id_shop, $lang = false, $search)
    {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__)) {
            return call_user_func_array(array(
                'ShopymindClient_CallbackOverride',
                __FUNCTION__
            ), func_get_args());
        }

        if (strlen($search) < self::SEARCH_MIN_LENGTH) {
            return array();
        }
        $attribute = Mage::getModel('eav/config')->getAttribute('catalog_product', SPM_ShopyMind_Helper_Data::MANUFACTURER_ATTRIBUTE_CODE);

        $scope = SPM_ShopyMind_Model_Scope::fromShopymindId($id_shop, $lang);
        $scope->restrictEavAttribute($attribute);

        $toShopyMindFormat = function($optionData) {
            return array(
                'id' => $optionData['value'],
                'name' => $optionData['label'],
            );
        };
        $matchesSearch = function($option) use ($search) {
            return stripos($option['name'], $search) !== false;
        };
        $options = array_filter(
            array_map($toShopyMindFormat, $attribute->getSource()->getAllOptions(false)),
            $matchesSearch
        );

        usort($options, function($optA, $optB) {
            return strcmp($optA['name'], $optB['name']);
        });
        return $options;
    }

    private static function returnCollectionDataOrCount(Varien_Data_Collection $collection, $justCount)
    {
        if ($justCount) {
            return self::counterResponse($collection);
        }

        return array_map(function ($customer) {
            return array('customer' => ShopymindClient_Callback::getUser($customer['entity_id']));
        }, $collection->getData());
    }

    /**
     * Allows to get customer emails from database.
     * This method is used to get emails when enabling retargeting. It will allow to know if the emails is real or not to not flag visitor as anonymous.
     * Once email is fetched, it will be hashed with md5
     *
     * @param string $id_shop storeId
     * @param int $start offset the result from the query
     * @param int $limit limit the result from the query.
     * @param string $lastUpdate date|datetime to narrow the query.
     *
     * @return array empty | array($customerId => $customerEmail);
     */
    public static function getExistingEmails($id_shop = null, $start = 0, $limit = null, $lastUpdate = null)
    {
        //Do we still need this ? It's looks like Prestashop Override mechanism
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__)) {
            return call_user_func_array(array(
                'ShopymindClient_CallbackOverride',
                __FUNCTION__
            ), func_get_args());
        }

        $scope = SPM_ShopyMind_Model_Scope::fromShopymindId($id_shop);

        $customersCollection = Mage::getModel('customer/customer')->getCollection()
            ->addAttributeToSelect('email')
            ->addAttributeToSelect('entity_id');

        $scope->restrictCollection($customersCollection);

        if (!is_null($lastUpdate)) {
            $customersCollection->addFieldToFilter('updated_at', array('gteq' => $lastUpdate));
        }

        if (!is_null($limit)) {
            $customersCollection->getSelect()->limit($limit, $start);
        }

        $customers = array();
        foreach ($customersCollection as $customer) {
            $customers[$customer->getId()] = $customer->getEmail();
        }

        //Add guest customer
        $guestCollection = Mage::getResourceModel('sales/order_collection')
        ->addAttributeToSelect('customer_email');

        $scope->restrictCollection($guestCollection);

        if (!is_null($lastUpdate)) {
            $guestCollection->addFieldToFilter('updated_at', array('gteq' => $lastUpdate));
        }

        $guestCollection->addFieldToFilter('customer_is_guest', 1);

        if (!is_null($limit)) {
            $guestCollection->getSelect()->limit($limit, $start);
        }


        foreach ($guestCollection as $customer) {
            if(!in_array($customer->getCustomerEmail(),$customers))
                $customers[$customer->getCustomerEmail()] = $customer->getCustomerEmail();
        }

        return $customers;
    }

   
    private static function restrictOrdersCollectionBillingAddressesToTimezones(Varien_Data_Collection_Db $collection, array $timezones)
    {
        $timezonesWhere = self::generateTimezonesWhere($timezones, 'sales/order_address', 'country_id', 'directory/country_region', 'code');
        if (!empty($timezonesWhere)) {
            $collection->getSelect()
                ->joinLeft(array('sales/order_address' => Mage::getSingleton('core/resource')->getTableName('sales_flat_order_address')), '`sales/order_address`.parent_id = main_table.entity_id AND `sales/order_address`.address_type = "billing"', null)
                ->joinLeft(array('directory/country_region' => Mage::getSingleton('core/resource')->getTableName('directory_country_region')), '`directory/country_region`.region_id = `sales/order_address`.region_id', null)
				->where($timezonesWhere);
        }
    }

    public static function findProducts($id_shop, $lang = false, $search)
    {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__)) {
            return call_user_func_array(array(
                'ShopymindClient_CallbackOverride',
                __FUNCTION__
            ), func_get_args());
        }

        if (strlen($search) < self::SEARCH_MIN_LENGTH) {
            return array();
        }

        $collection = Mage::getModel('catalog/product')->getCollection();
        $collection
            ->addAttributeToSelect('name')
            ->addAttributeToFilter(array(
                array(
                    'attribute' => 'name',
                    'like' => '%' . $search . '%'
                ),
                array(
                    'attribute' => 'sku',
                    'like' => '%' . $search . '%'
                ),
            ))
            ->addAttributeToSort('name', Varien_Data_Collection_Db::SORT_ORDER_ASC)
            ->addAttributeToFilter('status', array(
                'in' => Mage::getModel('catalog/product_status')->getVisibleStatusIds())
            )
            ->addAttributeToFilter('visibility', array(
                'in' => Mage::getModel('catalog/product_visibility')->getVisibleInCatalogIds())
            )
            ->setPage(1, 100) // limit to prevent abuses
        ;

        $scope = SPM_ShopyMind_Model_Scope::fromShopymindId($id_shop, $lang);
        $scope->restrictProductCollection($collection);

        $products = array();
        foreach ($collection as $product) {
            /** @var Mage_Catalog_Model_Product $product */
            $data = array(
                'id' => $product->getId(),
                'name' => $product->getName(),
                'combinations' => self::combinationsOfProduct($product)
            );
            $products[] = $data;
        }
        return $products;
    }

    private static function combinationsOfProduct(Mage_Catalog_Model_Product $product)
    {
        $formatter = function ($childProduct) {
            return array(
                'id' => $childProduct->getId(),
                'name' => $childProduct->getName()
            );
        };
        return Mage::helper('shopymind')->formatCombinationsOfProduct($product, $formatter, array('name'));
    }

    public static function findCategories($id_shop, $lang = false, $search) {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__)) {
            return call_user_func_array(array(
                'ShopymindClient_CallbackOverride',
                __FUNCTION__
            ), func_get_args());
        }

        $scope = SPM_ShopyMind_Model_Scope::fromShopymindId($id_shop);
        if ($lang) {
            self::startStoreEmulationByIsoLang($lang,$id_shop);
        } elseif ($id_shop) {
            $storeIds = $scope->storeIds();
            if (!empty($storeIds)) {
                self::startStoreEmulationByStoreId($storeIds[0]);
            }
        } else {
           self::startAdminStoreEmulation();
        }

        if (strlen($search) < self::SEARCH_MIN_LENGTH) {
            return array();
        }

        $collection = Mage::getModel('catalog/category')->getCollection();
        $collection
            ->addAttributeToSelect('name')
            ->addAttributeToFilter('name', array('like' => '%' . $search . '%'))
            ->addIsActiveFilter()
            ->addOrderField('name')
        ;

        $scope->restrictCategoryCollection($collection);

        $categories = array();
        foreach($collection as $category) {
            $categories[] = array(
                'id' => $category->getId(),
                'name' => $category->getName()
            );
        }

        self::stopStoreEmulation();

        return $categories;
    }


    /**
     * Returns the details of the requested cart(s)
     *
     * @param integer $id_shop Shop id
     * @param integer|array $id_cart Cart id(s) to get details from
     *
     * @return array
     */
    public static function getCarts($id_shop, $id_cart)
    {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__)) {
            return call_user_func_array(array (
                'ShopymindClient_CallbackOverride',
                __FUNCTION__
            ), func_get_args());
        }

        $collection = Mage::getResourceModel('sales/quote_collection')
            ->addFieldToFilter('entity_id', array('in' => (array) $id_cart));

        $scope = SPM_ShopyMind_Model_Scope::fromShopymindId($id_shop);
        $scope->restrictCollection($collection);

        $results = array();

        foreach ($collection as $quote) {
            $quoteData = $quote->getData();
            
            if($quoteData['store_id'])
            $currentScope = SPM_ShopyMind_Model_Scope::fromShopymindId('store-'.$quoteData['store_id']);
            else $currentScope = $scope;
			$cartProducts = self::productsOfCart($currentScope, ($quoteData['store_id']) ? $quoteData['store_id'] : null, $quote->getId());
            $currency = ($quote->getQuoteCurrencyCode() !== null && $quote->getQuoteCurrencyCode() ? $quote->getQuoteCurrencyCode() : ($quote->getStoreCurrencyCode() !== null && $quote->getStoreCurrencyCode() ? $quote->getStoreCurrencyCode() : $quote->getBaseCurrencyCode()));
            $currency_rate = ($quote->getStoreToQuoteRate() ? $quote->getStoreToQuoteRate() : $quote->getStoreToBaseRate());
            $results[] = array(
                'sum_cart' => ($quote->getBaseGrandTotal() * $currency_rate),
                'currency' => $currency,
                'tax_rate' => $currency_rate,
                'id_cart' => $quote->getId(),
                'date_cart' =>$quote->getCreatedAt(),
                'date_upd' => $quote->getUpdatedAt(),
                'link_cart' => Mage::helper('shopymind')->getUrl('checkout/cart', array(
        			'_current' => false,
        			'_use_rewrite' => true,
        			'_secure' => true,
        			'_store' => ($quoteData['store_id']) ? $quoteData['store_id'] : null,
        			'_store_to_url' => false,
        			'_nosid' => true
        		)),
                'products' => $cartProducts,
                'customer' => self::getUser(($quote->getCustomerId() ? $quote->getCustomerId() : $quote->getCustomerEmail()), true)
            );
        }

        return $results;
    }


    /**
     * Create customer
     *
     * @param integer $id_shop
     * @param string $lang
     * @param array $customer_infos
     * @return boolean|array
     */
    public static function createCustomer($id_shop, $customer_infos, $lang = null)
    {

        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__)) {
            return call_user_func_array(array (
                'ShopymindClient_CallbackOverride',
                __FUNCTION__
            ), func_get_args());
        }
        $return = true;

        $scope = SPM_ShopyMind_Model_Scope::fromShopymindId($id_shop,$lang);
        $stores = $scope->stores();

        if(is_array($stores) && count($stores)) {
            $store = $stores[0];
        }
        else {
            $scope = SPM_ShopyMind_Model_Scope::fromShopymindId($id_shop);
            $stores = $scope->stores();
            if(is_array($stores) && count($stores))
                $store = $stores[0];
            else $return = false;
        }
        if($return) {
			$customer = Mage::getModel('customer/customer');
			$customer->setWebsiteId(Mage::app()->getWebsite()->getId());
			$customer->loadByEmail($customer_infos['email']);

			if($customer->getId())
			{
				return array(
					'success' 	=> false,
					'message'	=> 'account_exists',
				);
			}
			
            $customer = Mage::getModel("customer/customer");
            $customer
            ->setFirstname($customer_infos['firstName'])
            ->setLastname($customer_infos['lastName'])
            ->setEmail($customer_infos['email'])
            ->setPassword($customer_infos['password']);
			
			if(array_key_exists('birthday', $customer_infos) && $customer_infos['birthday'] != '' && $customer_infos['birthday'] != '0000-00-00') {
				$datetime = DateTime::createFromFormat('Y-m-d', $customer_infos['birthday']);
				$customer->setDob($datetime->format('m-d-Y'));
			}
			else {
				$customer->setDob(null);
			}
			
			if(array_key_exists('gender', $customer_infos)) {
				$customer->setGender(($customer_infos['gender'] == 'mr'  ? 1 : 2));
			}
			else {
				$customer->setGender(0);
			}

            $customer
            ->setStore($store)
            ->setWebsiteId($store->getWebsiteId());

            try{
                $customer->save();
                $customer->setConfirmation(null);
                $customer->save();
                $customer->sendNewAccountEmail('registered','',$store->getId());

                // Inscription à la newsletter si necessaire
                if ((int)$customer_infos['newsletter'] && $customer->getId()){
                    // load up the subscriber if possible
                    $subscriber = Mage::getModel('newsletter/subscriber')->loadByEmail($customer_infos['email']);
                    if (!$subscriber->getId()
                        || $subscriber->getStatus() == Mage_Newsletter_Model_Subscriber::STATUS_UNSUBSCRIBED
                        || $subscriber->getStatus() == Mage_Newsletter_Model_Subscriber::STATUS_NOT_ACTIVE) {
                        $subscriber->setStatus(Mage_Newsletter_Model_Subscriber::STATUS_SUBSCRIBED);
                        $subscriber->setSubscriberEmail($customer_infos['email']);
                        $subscriber->setSubscriberConfirmCode($subscriber->RandomSequence());
                    }
                    $subscriber->setStoreId(Mage::app()->getStore()->getId());
                    $subscriber->setCustomerId($customer->getId());
                    try {
                        $subscriber->save();
                    }
                    catch (Exception $e) {
                        Mage::log($e->getMessage(), Zend_Log::ERR);
                        $return = false;
                    }
                }
            }
            catch (Exception $e) {
                Mage::log($e->getMessage(), Zend_Log::ERR);
                $return = false;
            }
        }

        return ($return ? self::getUser($customer->getId()) : false);
    }

    public static function getOrder($idOrder)
    {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__)) {
            return call_user_func_array(array (
                'ShopymindClient_CallbackOverride',
                __FUNCTION__
            ), func_get_args());
        }
        return self::formatOrderData(Mage::getModel('sales/order')->load($idOrder));
    }

    public static function syncCustomers($id_shop, $start, $limit, $lastUpdate, $idCustomer = false, $justCount = false)
    {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__)) {
            return call_user_func_array(array (
                'ShopymindClient_CallbackOverride',
                __FUNCTION__
            ), func_get_args());
        }

        $scope = SPM_ShopyMind_Model_Scope::fromShopymindId($id_shop);
        $SyncCustomersAction = new SPM_ShopyMind_Action_SyncCustomers($scope, $start, $limit, $lastUpdate, $idCustomer, $justCount);
        return $SyncCustomersAction->process();
    }

    public static function syncCustomersGroups($id_shop, $start, $limit, $lastUpdate, $id_group = false, $justCount = false)
    {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__)) {
            return call_user_func_array(array (
                'ShopymindClient_CallbackOverride',
                __FUNCTION__
            ), func_get_args());
        }

        $scope = SPM_ShopyMind_Model_Scope::fromShopymindId($id_shop);
        $SyncCustomersAction = new SPM_ShopyMind_Action_SyncCustomersGroups($scope, $start, $limit, $lastUpdate, $id_group, $justCount);
        return $SyncCustomersAction->process();
    }
    public static function syncVouchers($id_shop, $start, $limit, $lastUpdate, $id_cart_rule = false, $justCount = false)
    {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__)) {
            return call_user_func_array(array (
                'ShopymindClient_CallbackOverride',
                __FUNCTION__
            ), func_get_args());
        }

        $scope = SPM_ShopyMind_Model_Scope::fromShopymindId($id_shop);
        $SyncCustomersAction = new SPM_ShopyMind_Action_SyncVouchers($scope, $start, $limit, $lastUpdate, $id_cart_rule, $justCount);
        return $SyncCustomersAction->process();
    }
    public static function syncOrdersStatus($id_shop, $start, $limit, $lastUpdate, $id_status = false, $justCount = false)
    {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__)) {
            return call_user_func_array(array (
                'ShopymindClient_CallbackOverride',
                __FUNCTION__
            ), func_get_args());
        }
        $scope = SPM_ShopyMind_Model_Scope::fromShopymindId($id_shop);
        $SyncCustomersAction = new SPM_ShopyMind_Action_SyncOrdersStatus($scope, $start, $limit, $lastUpdate, $id_status, $justCount);
        return $SyncCustomersAction->process();
    }

    public static function syncCarriers($id_shop, $start, $limit, $lastUpdate, $id_carrier = false, $justCount = false)
    {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__)) {
            return call_user_func_array(array (
                'ShopymindClient_CallbackOverride',
                __FUNCTION__
            ), func_get_args());
        }
        $scope = SPM_ShopyMind_Model_Scope::fromShopymindId($id_shop);
        $SyncCustomersAction = new SPM_ShopyMind_Action_SyncCarriers($scope, $start, $limit, $lastUpdate, $id_carrier, $justCount);
        return $SyncCustomersAction->process();
    }
    public static function syncManufacturers($id_shop, $start, $limit, $lastUpdate, $id_manufacturer = false, $justCount = false)
    {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__)) {
            return call_user_func_array(array (
                'ShopymindClient_CallbackOverride',
                __FUNCTION__
            ), func_get_args());
        }
        $scope = SPM_ShopyMind_Model_Scope::fromShopymindId($id_shop);
        $SyncCustomersAction = new SPM_ShopyMind_Action_SyncManufacturers($scope, $start, $limit, $lastUpdate, $id_manufacturer, $justCount);
        return $SyncCustomersAction->process();
    }

    public static function syncProducts($id_shop, $start, $limit, $lastUpdate, $idProduct= false, $justCount = false)
    {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__)) {
            return call_user_func_array(array (
                'ShopymindClient_CallbackOverride',
                __FUNCTION__
            ), func_get_args());
        }

        $scope = SPM_ShopyMind_Model_Scope::fromShopymindId($id_shop);
        $SyncProductsAction = new SPM_ShopyMind_Action_SyncProducts($scope, $start, $limit, $lastUpdate, $idProduct, $justCount);
        return $SyncProductsAction->process();
    }

    public static function syncProductsCategories($id_shop, $start, $limit, $lastUpdate, $idCategory = false, $justCount = false)
    {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__)) {
            return call_user_func_array(array (
                'ShopymindClient_CallbackOverride',
                __FUNCTION__
            ), func_get_args());
        }

        $scope = SPM_ShopyMind_Model_Scope::fromShopymindId($id_shop);
        $SyncProductCategoriesAction = new SPM_ShopyMind_Action_SyncProductCategories($scope, $start, $limit, $lastUpdate, $idCategory, $justCount);
        return $SyncProductCategoriesAction->process();
    }

    public static function syncOrders($id_shop, $start, $limit, $lastUpdate, $orderId = false, $justCount = false)
    {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__)) {
            return call_user_func_array(array (
                'ShopymindClient_CallbackOverride',
                __FUNCTION__
            ), func_get_args());
        }

        $scope = SPM_ShopyMind_Model_Scope::fromShopymindId($id_shop);
        $SyncCustomersAction = new SPM_ShopyMind_Action_SyncOrders($scope, $start, $limit, $lastUpdate, $orderId, $justCount);
        return $SyncCustomersAction->process();
    }

    public static function syncNLSubscribers($id_shop, $start, $limit, $lastUpdate, $justCount = false)
    {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__)) {
            return call_user_func_array(array (
                'ShopymindClient_CallbackOverride',
                __FUNCTION__
            ), func_get_args());
        }

        $scope = SPM_ShopyMind_Model_Scope::fromShopymindId($id_shop);

        // If subscribers is activated, we retrieve subscribers data
        $activation = $scope->getConfig('shopymind/configuration/syncnewsletters');
        if($activation == '1') {
            $SyncNewslettersAction = new SPM_ShopyMind_Action_SyncNlSubscribers($scope, $start, $limit, $lastUpdate, $justCount);
            return $SyncNewslettersAction->process();
        }
        else {
            return array();
        }
    }

    /**
     * Permet de désinscrire un customer de la newsletter
     * @param $id_customer
     * @return bool|mixed|string
     */
    public static function unsubCustomer($id_shop, $id_customer, $email)
    {
        if (class_exists('ShopymindClient_CallbackOverride', false) && method_exists('ShopymindClient_CallbackOverride', __FUNCTION__)) {
            return call_user_func_array(array(
                'ShopymindClient_CallbackOverride',
                __FUNCTION__
            ), func_get_args());
        }

        if (is_null($id_customer) || !$id_customer) {
            return false;
        }

        if (is_numeric($id_customer)) {
			$scope = SPM_ShopyMind_Model_Scope::fromShopymindId($id_shop);
			
			$storeIds = $scope->storeIds();
            if (!empty($storeIds)) {
                self::startStoreEmulationByStoreId($storeIds[0]);
			
				$tablePrefix = Mage::getConfig()->getTablePrefix();
				$read = Mage::getSingleton('core/resource')->getConnection('core_write');

				try {
					$result = $read->query('UPDATE `' . $tablePrefix . 'newsletter_subscriber`
					SET `subscriber_status` = 3 WHERE `customer_id` = ' . (int)$id_customer.' AND `store_id` = '.$storeIds[0]);
				}
				catch(\Exception $e) {
					self::stopStoreEmulation();
					return false;
				}
				
				self::stopStoreEmulation();
            }
        }

        return true;
    }

    /**
     * Permet de changer l'url du script spm en bas de page
     * @param $id_customer
     * @return bool|mixed|string
     */
    public static function setScriptUrl($new_script_url)
    {
        if($new_script_url == '' || !$new_script_url || $new_script_url == "0") {
            $new_script_url = "https://app-spm.com";
        }
        Mage::getModel('core/config')->saveConfig('shopimind/script_url', $new_script_url);
        return true;
    }
}
